/* Copyright (C) 2000  Free Software Foundation, Inc.

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you link this library with other files,
   some of which are compiled with GCC, to produce an executable,
   this library does not by itself cause the resulting executable
   to be covered by the GNU General Public License.
   This exception does not however invalidate any other reasons why
   the executable file might be covered by the GNU General Public License.  */

/* stdlib.h - Definitions for common types, variables, and functions.  */

#if !defined(__STDLIB)
#define __STDLIB

#ifndef _STDDDEF_H_
#include <stddef.h>
#endif

#ifndef EDOM
#if 0
#include <errno.h>
#else
#define	EDOM		33		/* Argument too large.  */
#endif
#endif

double atof (const char *s);
int atoi (const char *s);
long atol (const char *s);
double strtod (const char *s, char **endptr);
long strtol (const char *s, char **endptr, int radix);
unsigned long strtoul (const char *s, char **endptr, int radix);
int rand (void);
void srand (unsigned seed);
void * calloc (size_t nitems, size_t size);
void * malloc (size_t size);
void * realloc(void *block, size_t size);
void free (void *block);
void abort (void) __attribute__ ((noreturn));
void exit (int status);
typedef void (* atexit_t)(void);
int atexit (atexit_t func);
int system (const char *command);
char * getenv (const char *name);
void * bsearch(const void *key, const void *base, 
	       size_t nelem, size_t width,
	       int (*fcmp)(const void *, const void *));

void qsort (void *base, size_t nelem, size_t width,
	    int (*fcmp)(const void *, const void *));

#define EXIT_SUCCESS 0
#define EXIT_FAILURE (-1)

#define abs(x) __builtin_abs(x)

/* long abs(long) -- Makes an inline instruction. */
#define labs(x) __builtin_labs(x)

#ifndef _LDIV_T
#define _LDIV_T
typedef struct {
 long quot;
 long rem;
} ldiv_t;
#endif
#ifndef _DIV_T
#define _DIV_T
typedef struct {
 int quot;
 int rem;
} div_t;
#endif
extern div_t div (int numer, int denom);
extern __inline__ div_t div (int numer, int denom)
{
  div_t res;
  if (denom == 0) errno = EDOM;
  res.quot = numer / denom; res.rem = numer % denom;
  return res;
}

extern ldiv_t ldiv (long numer, long denom);
extern __inline__ ldiv_t ldiv (long numer, long denom)
{ 
  ldiv_t res;
  if (denom == 0) errno = EDOM;
  res.quot = numer / denom; res.rem = numer % denom;
  return res;
}

#endif
